@@ -23,28 +23,29 @@ module: s3_bucket
 version_added: 1.0.0
 short_description: Manage S3 buckets in AWS, DigitalOcean, Ceph, Walrus, FakeS3 and StorageGRID
 description:
-    - Manage S3 buckets in AWS, DigitalOcean, Ceph, Walrus, FakeS3 and StorageGRID
-requirements: [ boto3 ]
-author: "Rob White (@wimnat)"
+    - Manage S3 buckets in AWS, DigitalOcean, Ceph, Walrus, FakeS3 and StorageGRID.
+author:
+    - Rob White (@wimnat)
+    - Aubin Bikouo (@abikouo)
 options:
   force:
     description:
       - When trying to delete a bucket, delete all keys (including versions and delete markers)
-        in the bucket first (an s3 bucket must be empty for a successful deletion)
+        in the bucket first (an S3 bucket must be empty for a successful deletion).
     type: bool
     default: 'no'
   name:
     description:
-      - Name of the s3 bucket
+      - Name of the S3 bucket.
     required: true
     type: str
   policy:
     description:
-      - The JSON policy as a string.
+      - The JSON policy as a string. Set to the string C("null") to force the absence of a policy.
     type: json
   s3_url:
     description:
-      - S3 URL endpoint for usage with DigitalOcean, Ceph, Eucalyptus and fakes3 etc.
+      - S3 URL endpoint for usage with DigitalOcean, Ceph, Eucalyptus and FakeS3 etc.
       - Assumes AWS if not specified.
       - For Walrus, use FQDN of the endpoint without scheme nor path.
     aliases: [ S3_URL ]
@@ -54,6 +55,7 @@ options:
       - Enable API compatibility with Ceph. It takes into account the S3 API subset working
         with Ceph in order to provide the same module behaviour where possible.
     type: bool
+    default: false
   requester_pays:
     description:
       - With Requester Pays buckets, the requester instead of the bucket owner pays the cost
@@ -61,23 +63,23 @@ options:
     type: bool
   state:
     description:
-      - Create or remove the s3 bucket
+      - Create or remove the S3 bucket.
     required: false
     default: present
     choices: [ 'present', 'absent' ]
     type: str
   tags:
     description:
-      - tags dict to apply to bucket
+      - Tags dict to apply to bucket.
     type: dict
   purge_tags:
     description:
-      - whether to remove tags that aren't present in the C(tags) parameter
+      - Whether to remove tags that aren't present in the I(tags) parameter.
     type: bool
     default: True
   versioning:
     description:
-      - Whether versioning is enabled or disabled (note that once versioning is enabled, it can only be suspended)
+      - Whether versioning is enabled or disabled (note that once versioning is enabled, it can only be suspended).
     type: bool
   encryption:
     description:
@@ -86,9 +88,60 @@ options:
     choices: [ 'none', 'AES256', 'aws:kms' ]
     type: str
   encryption_key_id:
-    description: KMS master key ID to use for the default encryption. This parameter is allowed if encryption is aws:kms. If
+    description: KMS master key ID to use for the default encryption. This parameter is allowed if I(encryption) is C(aws:kms). If
                  not specified then it will default to the AWS provided KMS key.
     type: str
+  public_access:
+    description:
+      - Configure public access block for S3 bucket.
+      - This option cannot be used together with I(delete_public_access).
+    suboptions:
+      block_public_acls:
+        description: Sets BlockPublicAcls value.
+        type: bool
+        default: False
+      block_public_policy:
+        description: Sets BlockPublicPolicy value.
+        type: bool
+        default: False
+      ignore_public_acls:
+        description: Sets IgnorePublicAcls value.
+        type: bool
+        default: False
+      restrict_public_buckets:
+        description: Sets RestrictPublicAcls value.
+        type: bool
+        default: False
+    type: dict
+    version_added: 1.3.0
+  delete_public_access:
+    description:
+      - Delete public access block configuration from bucket.
+      - This option cannot be used together with a I(public_access) definition.
+    default: false
+    type: bool
+    version_added: 1.3.0
+  object_ownership:
+    description:
+      - Allow bucket's ownership controls.
+      - C(BucketOwnerPreferred) - Objects uploaded to the bucket change ownership to the bucket owner
+        if the objects are uploaded with the bucket-owner-full-control canned ACL.
+      - C(ObjectWriter) - The uploading account will own the object
+        if the object is uploaded with the bucket-owner-full-control canned ACL.
+      - This option cannot be used together with a I(delete_object_ownership) definition.
+      - Management of bucket ownership controls requires botocore>=1.18.11.
+    choices: [ 'BucketOwnerPreferred', 'ObjectWriter' ]
+    type: str
+    version_added: 2.0.0
+  delete_object_ownership:
+    description:
+      - Delete bucket's ownership controls.
+      - This option cannot be used together with a I(object_ownership) definition.
+      - Management of bucket ownership controls requires botocore>=1.18.11.
+    default: false
+    type: bool
+    version_added: 2.0.0
+
 extends_documentation_fragment:
 - amazon.aws.aws
 - amazon.aws.ec2
@@ -103,18 +156,18 @@ notes:
 EXAMPLES = '''
 # Note: These examples do not set authentication details, see the AWS Guide for details.
 
-# Create a simple s3 bucket
+# Create a simple S3 bucket
 - amazon.aws.s3_bucket:
     name: mys3bucket
     state: present
 
-# Create a simple s3 bucket on Ceph Rados Gateway
+# Create a simple S3 bucket on Ceph Rados Gateway
 - amazon.aws.s3_bucket:
     name: mys3bucket
     s3_url: http://your-ceph-rados-gateway-server.xxx
     ceph: true
 
-# Remove an s3 bucket and any keys it contains
+# Remove an S3 bucket and any keys it contains
 - amazon.aws.s3_bucket:
     name: mys3bucket
     state: absent
@@ -153,6 +206,41 @@ EXAMPLES = '''
     name: mys3bucket
     state: present
     encryption: "aws:kms"
+
+# Create a bucket with public policy block configuration
+- amazon.aws.s3_bucket:
+    name: mys3bucket
+    state: present
+    public_access:
+        block_public_acls: true
+        ignore_public_acls: true
+        ## keys == 'false' can be omitted, undefined keys defaults to 'false'
+        # block_public_policy: false
+        # restrict_public_buckets: false
+
+# Delete public policy block from bucket
+- amazon.aws.s3_bucket:
+    name: mys3bucket
+    state: present
+    delete_public_access: true
+
+# Create a bucket with object ownership controls set to ObjectWriter
+- amazon.aws.s3_bucket:
+    name: mys3bucket
+    state: present
+    object_ownership: ObjectWriter
+
+# Delete onwership controls from bucket
+- amazon.aws.s3_bucket:
+    name: mys3bucket
+    state: present
+    delete_object_ownership: true
+
+# Delete a bucket policy from bucket
+- amazon.aws.s3_bucket:
+    name: mys3bucket
+    state: present
+    policy: "null"
 '''
 
 import json
@@ -160,7 +248,7 @@ import os
 import time
 
 try:
-    from botocore.exceptions import BotoCoreError, ClientError, EndpointConnectionError, WaiterError
+    import botocore
 except ImportError:
     pass  # Handled by AnsibleAWSModule
 
@@ -176,6 +264,8 @@ from ..module_utils.ec2 import boto3_conn
 from ..module_utils.ec2 import boto3_tag_list_to_ansible_dict
 from ..module_utils.ec2 import compare_policies
 from ..module_utils.ec2 import get_aws_connection_info
+from ..module_utils.ec2 import snake_dict_to_camel_dict
+from ..module_utils.s3 import validate_bucket_name
 
 
 def create_or_update_bucket(s3_client, module, location):
@@ -188,14 +278,18 @@ def create_or_update_bucket(s3_client, module, location):
     versioning = module.params.get("versioning")
     encryption = module.params.get("encryption")
     encryption_key_id = module.params.get("encryption_key_id")
+    public_access = module.params.get("public_access")
+    delete_public_access = module.params.get("delete_public_access")
+    delete_object_ownership = module.params.get("delete_object_ownership")
+    object_ownership = module.params.get("object_ownership")
     changed = False
     result = {}
 
     try:
         bucket_is_present = bucket_exists(s3_client, name)
-    except EndpointConnectionError as e:
+    except botocore.exceptions.EndpointConnectionError as e:
         module.fail_json_aws(e, msg="Invalid endpoint provided: %s" % to_text(e))
-    except (BotoCoreError, ClientError) as e:
+    except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
         module.fail_json_aws(e, msg="Failed to check bucket presence")
 
     if not bucket_is_present:
@@ -203,19 +297,19 @@ def create_or_update_bucket(s3_client, module, location):
             bucket_changed = create_bucket(s3_client, name, location)
             s3_client.get_waiter('bucket_exists').wait(Bucket=name)
             changed = changed or bucket_changed
-        except WaiterError as e:
+        except botocore.exceptions.WaiterError as e:
             module.fail_json_aws(e, msg='An error occurred waiting for the bucket to become available')
-        except (BotoCoreError, ClientError) as e:
+        except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
             module.fail_json_aws(e, msg="Failed while creating bucket")
 
     # Versioning
     try:
         versioning_status = get_bucket_versioning(s3_client, name)
-    except is_boto3_error_code(['NotImplemented', 'XNotImplemented']) as exp:
+    except is_boto3_error_code(['NotImplemented', 'XNotImplemented']) as e:
         if versioning is not None:
-            module.fail_json_aws(exp, msg="Failed to get bucket versioning")
-    except (BotoCoreError, ClientError) as exp:
-        module.fail_json_aws(exp, msg="Failed to get bucket versioning")
+            module.fail_json_aws(e, msg="Failed to get bucket versioning")
+    except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:  # pylint: disable=duplicate-except
+        module.fail_json_aws(e, msg="Failed to get bucket versioning")
     else:
         if versioning is not None:
             required_versioning = None
@@ -228,7 +322,7 @@ def create_or_update_bucket(s3_client, module, location):
                 try:
                     put_bucket_versioning(s3_client, name, required_versioning)
                     changed = True
-                except (BotoCoreError, ClientError) as e:
+                except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
                     module.fail_json_aws(e, msg="Failed to update bucket versioning")
 
                 versioning_status = wait_versioning_is_applied(module, s3_client, name, required_versioning)
@@ -242,11 +336,11 @@ def create_or_update_bucket(s3_client, module, location):
     # Requester pays
     try:
         requester_pays_status = get_bucket_request_payment(s3_client, name)
-    except is_boto3_error_code(['NotImplemented', 'XNotImplemented']):
+    except is_boto3_error_code(['NotImplemented', 'XNotImplemented']) as e:
         if requester_pays is not None:
-            module.fail_json_aws(exp, msg="Failed to get bucket request payment")
-    except (BotoCoreError, ClientError) as exp:
-        module.fail_json_aws(exp, msg="Failed to get bucket request payment")
+            module.fail_json_aws(e, msg="Failed to get bucket request payment")
+    except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:  # pylint: disable=duplicate-except
+        module.fail_json_aws(e, msg="Failed to get bucket request payment")
     else:
         if requester_pays is not None:
             payer = 'Requester' if requester_pays else 'BucketOwner'
@@ -265,11 +359,11 @@ def create_or_update_bucket(s3_client, module, location):
     # Policy
     try:
         current_policy = get_bucket_policy(s3_client, name)
-    except is_boto3_error_code(['NotImplemented', 'XNotImplemented']):
+    except is_boto3_error_code(['NotImplemented', 'XNotImplemented']) as e:
         if policy is not None:
-            module.fail_json_aws(exp, msg="Failed to get bucket policy")
-    except (BotoCoreError, ClientError) as exp:
-        module.fail_json_aws(exp, msg="Failed to get bucket policy")
+            module.fail_json_aws(e, msg="Failed to get bucket policy")
+    except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:  # pylint: disable=duplicate-except
+        module.fail_json_aws(e, msg="Failed to get bucket policy")
     else:
         if policy is not None:
             if isinstance(policy, string_types):
@@ -278,14 +372,14 @@ def create_or_update_bucket(s3_client, module, location):
             if not policy and current_policy:
                 try:
                     delete_bucket_policy(s3_client, name)
-                except (BotoCoreError, ClientError) as e:
+                except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
                     module.fail_json_aws(e, msg="Failed to delete bucket policy")
                 current_policy = wait_policy_is_applied(module, s3_client, name, policy)
                 changed = True
             elif compare_policies(current_policy, policy):
                 try:
                     put_bucket_policy(s3_client, name, policy)
-                except (BotoCoreError, ClientError) as e:
+                except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
                     module.fail_json_aws(e, msg="Failed to update bucket policy")
                 current_policy = wait_policy_is_applied(module, s3_client, name, policy, should_fail=False)
                 if current_policy is None:
@@ -300,11 +394,11 @@ def create_or_update_bucket(s3_client, module, location):
     # Tags
     try:
         current_tags_dict = get_current_bucket_tags_dict(s3_client, name)
-    except is_boto3_error_code(['NotImplemented', 'XNotImplemented']):
+    except is_boto3_error_code(['NotImplemented', 'XNotImplemented']) as e:
         if tags is not None:
-            module.fail_json_aws(exp, msg="Failed to get bucket tags")
-    except (ClientError, BotoCoreError) as exp:
-        module.fail_json_aws(exp, msg="Failed to get bucket tags")
+            module.fail_json_aws(e, msg="Failed to get bucket tags")
+    except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:  # pylint: disable=duplicate-except
+        module.fail_json_aws(e, msg="Failed to get bucket tags")
     else:
         if tags is not None:
             # Tags are always returned as text
@@ -318,13 +412,13 @@ def create_or_update_bucket(s3_client, module, location):
                 if tags:
                     try:
                         put_bucket_tagging(s3_client, name, tags)
-                    except (BotoCoreError, ClientError) as e:
+                    except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
                         module.fail_json_aws(e, msg="Failed to update bucket tags")
                 else:
                     if purge_tags:
                         try:
                             delete_bucket_tagging(s3_client, name)
-                        except (BotoCoreError, ClientError) as e:
+                        except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
                             module.fail_json_aws(e, msg="Failed to delete bucket tags")
                 current_tags_dict = wait_tags_are_applied(module, s3_client, name, tags)
                 changed = True
@@ -334,36 +428,104 @@ def create_or_update_bucket(s3_client, module, location):
     # Encryption
     try:
         current_encryption = get_bucket_encryption(s3_client, name)
-    except (ClientError, BotoCoreError) as e:
-        module.fail_json_aws(e, msg="Failed to get bucket encryption")
-
-    if encryption is not None:
-        current_encryption_algorithm = current_encryption.get('SSEAlgorithm') if current_encryption else None
-        current_encryption_key = current_encryption.get('KMSMasterKeyID') if current_encryption else None
-        if encryption == 'none' and current_encryption_algorithm is not None:
-            try:
-                delete_bucket_encryption(s3_client, name)
-            except (BotoCoreError, ClientError) as e:
-                module.fail_json_aws(e, msg="Failed to delete bucket encryption")
-            current_encryption = wait_encryption_is_applied(module, s3_client, name, None)
-            changed = True
-        elif encryption != 'none' and (encryption != current_encryption_algorithm) or (encryption == 'aws:kms' and current_encryption_key != encryption_key_id):
-            expected_encryption = {'SSEAlgorithm': encryption}
-            if encryption == 'aws:kms' and encryption_key_id is not None:
-                expected_encryption.update({'KMSMasterKeyID': encryption_key_id})
-            current_encryption = put_bucket_encryption_with_retry(module, s3_client, name, expected_encryption)
-            changed = True
+    except is_boto3_error_code(['NotImplemented', 'XNotImplemented']) as e:
+        if encryption is not None:
+            module.fail_json_aws(e, msg="Failed to get bucket encryption settings")
+    except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:  # pylint: disable=duplicate-except
+        module.fail_json_aws(e, msg="Failed to get bucket encryption settings")
+    else:
+        if encryption is not None:
+            current_encryption_algorithm = current_encryption.get('SSEAlgorithm') if current_encryption else None
+            current_encryption_key = current_encryption.get('KMSMasterKeyID') if current_encryption else None
+            if encryption == 'none':
+                if current_encryption_algorithm is not None:
+                    try:
+                        delete_bucket_encryption(s3_client, name)
+                    except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
+                        module.fail_json_aws(e, msg="Failed to delete bucket encryption")
+                    current_encryption = wait_encryption_is_applied(module, s3_client, name, None)
+                    changed = True
+            else:
+                if (encryption != current_encryption_algorithm) or (encryption == 'aws:kms' and current_encryption_key != encryption_key_id):
+                    expected_encryption = {'SSEAlgorithm': encryption}
+                    if encryption == 'aws:kms' and encryption_key_id is not None:
+                        expected_encryption.update({'KMSMasterKeyID': encryption_key_id})
+                    current_encryption = put_bucket_encryption_with_retry(module, s3_client, name, expected_encryption)
+                    changed = True
 
         result['encryption'] = current_encryption
 
+    # Public access clock configuration
+    current_public_access = {}
+
+    try:
+        current_public_access = get_bucket_public_access(s3_client, name)
+    except is_boto3_error_code(['NotImplemented', 'XNotImplemented']) as e:
+        if public_access is not None:
+            module.fail_json_aws(e, msg="Failed to get bucket public access configuration")
+    except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:  # pylint: disable=duplicate-except
+        module.fail_json_aws(e, msg="Failed to get bucket public access configuration")
+    else:
+        # -- Create / Update public access block
+        if public_access is not None:
+            camel_public_block = snake_dict_to_camel_dict(public_access, capitalize_first=True)
+
+            if current_public_access == camel_public_block:
+                result['public_access_block'] = current_public_access
+            else:
+                put_bucket_public_access(s3_client, name, camel_public_block)
+                changed = True
+                result['public_access_block'] = camel_public_block
+
+        # -- Delete public access block
+        if delete_public_access:
+            if current_public_access == {}:
+                result['public_access_block'] = current_public_access
+            else:
+                delete_bucket_public_access(s3_client, name)
+                changed = True
+                result['public_access_block'] = {}
+
+    # -- Bucket ownership
+    try:
+        bucket_ownership = get_bucket_ownership_cntrl(s3_client, module, name)
+        result['object_ownership'] = bucket_ownership
+    except KeyError as e:
+        # Some non-AWS providers appear to return policy documents that aren't
+        # compatible with AWS, cleanly catch KeyError so users can continue to use
+        # other features.
+        if delete_object_ownership or object_ownership is not None:
+            module.fail_json_aws(e, msg="Failed to get bucket object ownership settings")
+    except is_boto3_error_code(['NotImplemented', 'XNotImplemented']) as e:
+        if delete_object_ownership or object_ownership is not None:
+            module.fail_json_aws(e, msg="Failed to get bucket object ownership settings")
+    except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:  # pylint: disable=duplicate-except
+        module.fail_json_aws(e, msg="Failed to get bucket bucket object ownership settings")
+    else:
+        if delete_object_ownership:
+            # delete S3 buckect ownership
+            if bucket_ownership is not None:
+                delete_bucket_ownership(s3_client, name)
+                changed = True
+                result['object_ownership'] = None
+        elif object_ownership is not None:
+            # update S3 bucket ownership
+            if bucket_ownership != object_ownership:
+                put_bucket_ownership(s3_client, name, object_ownership)
+                changed = True
+                result['object_ownership'] = object_ownership
+
+    # Module exit
     module.exit_json(changed=changed, name=name, **result)
 
 
 def bucket_exists(s3_client, bucket_name):
-    # head_bucket appeared to be really inconsistent, so we use list_buckets instead,
-    # and loop over all the buckets, even if we know it's less performant :(
-    all_buckets = s3_client.list_buckets(Bucket=bucket_name)['Buckets']
-    return any(bucket['Name'] == bucket_name for bucket in all_buckets)
+    try:
+        s3_client.head_bucket(Bucket=bucket_name)
+        bucket_exists = True
+    except is_boto3_error_code('404'):
+        bucket_exists = False
+    return bucket_exists
 
 
 @AWSRetry.exponential_backoff(max_delay=120)
@@ -430,9 +592,6 @@ def put_bucket_versioning(s3_client, bucket_name, required_versioning):
 
 @AWSRetry.exponential_backoff(max_delay=120, catch_extra_error_codes=['NoSuchBucket', 'OperationAborted'])
 def get_bucket_encryption(s3_client, bucket_name):
-    if not hasattr(s3_client, "get_bucket_encryption"):
-        return None
-
     try:
         result = s3_client.get_bucket_encryption(Bucket=bucket_name)
         return result.get('ServerSideEncryptionConfiguration', {}).get('Rules', [])[0].get('ApplyServerSideEncryptionByDefault')
@@ -447,7 +606,7 @@ def put_bucket_encryption_with_retry(module, s3_client, name, expected_encryptio
     for retries in range(1, max_retries + 1):
         try:
             put_bucket_encryption(s3_client, name, expected_encryption)
-        except (BotoCoreError, ClientError) as e:
+        except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:  # pylint: disable=duplicate-except
             module.fail_json_aws(e, msg="Failed to set bucket encryption")
         current_encryption = wait_encryption_is_applied(module, s3_client, name, expected_encryption,
                                                         should_fail=(retries == max_retries), retries=5)
@@ -487,11 +646,47 @@ def delete_bucket(s3_client, bucket_name):
         pass
 
 
+@AWSRetry.exponential_backoff(max_delay=120, catch_extra_error_codes=['NoSuchBucket', 'OperationAborted'])
+def put_bucket_public_access(s3_client, bucket_name, public_acces):
+    '''
+    Put new public access block to S3 bucket
+    '''
+    s3_client.put_public_access_block(Bucket=bucket_name, PublicAccessBlockConfiguration=public_acces)
+
+
+@AWSRetry.exponential_backoff(max_delay=120, catch_extra_error_codes=['NoSuchBucket', 'OperationAborted'])
+def delete_bucket_public_access(s3_client, bucket_name):
+    '''
+    Delete public access block from S3 bucket
+    '''
+    s3_client.delete_public_access_block(Bucket=bucket_name)
+
+
+@AWSRetry.exponential_backoff(max_delay=120, catch_extra_error_codes=['NoSuchBucket', 'OperationAborted'])
+def delete_bucket_ownership(s3_client, bucket_name):
+    '''
+    Delete bucket ownership controls from S3 bucket
+    '''
+    s3_client.delete_bucket_ownership_controls(Bucket=bucket_name)
+
+
+@AWSRetry.exponential_backoff(max_delay=120, catch_extra_error_codes=['NoSuchBucket', 'OperationAborted'])
+def put_bucket_ownership(s3_client, bucket_name, target):
+    '''
+    Put bucket ownership controls for S3 bucket
+    '''
+    s3_client.put_bucket_ownership_controls(
+        Bucket=bucket_name,
+        OwnershipControls={
+            'Rules': [{'ObjectOwnership': target}]
+        })
+
+
 def wait_policy_is_applied(module, s3_client, bucket_name, expected_policy, should_fail=True):
     for dummy in range(0, 12):
         try:
             current_policy = get_bucket_policy(s3_client, bucket_name)
-        except (ClientError, BotoCoreError) as e:
+        except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
             module.fail_json_aws(e, msg="Failed to get bucket policy")
 
         if compare_policies(current_policy, expected_policy):
@@ -509,7 +704,7 @@ def wait_payer_is_applied(module, s3_client, bucket_name, expected_payer, should
     for dummy in range(0, 12):
         try:
             requester_pays_status = get_bucket_request_payment(s3_client, bucket_name)
-        except (BotoCoreError, ClientError) as e:
+        except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
             module.fail_json_aws(e, msg="Failed to get bucket request payment")
         if requester_pays_status != expected_payer:
             time.sleep(5)
@@ -526,7 +721,7 @@ def wait_encryption_is_applied(module, s3_client, bucket_name, expected_encrypti
     for dummy in range(0, retries):
         try:
             encryption = get_bucket_encryption(s3_client, bucket_name)
-        except (BotoCoreError, ClientError) as e:
+        except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
             module.fail_json_aws(e, msg="Failed to get updated encryption for bucket")
         if encryption != expected_encryption:
             time.sleep(5)
@@ -544,7 +739,7 @@ def wait_versioning_is_applied(module, s3_client, bucket_name, required_versioni
     for dummy in range(0, 24):
         try:
             versioning_status = get_bucket_versioning(s3_client, bucket_name)
-        except (BotoCoreError, ClientError) as e:
+        except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
             module.fail_json_aws(e, msg="Failed to get updated versioning for bucket")
         if versioning_status.get('Status') != required_versioning:
             time.sleep(8)
@@ -558,7 +753,7 @@ def wait_tags_are_applied(module, s3_client, bucket_name, expected_tags_dict):
     for dummy in range(0, 12):
         try:
             current_tags_dict = get_current_bucket_tags_dict(s3_client, bucket_name)
-        except (ClientError, BotoCoreError) as e:
+        except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
             module.fail_json_aws(e, msg="Failed to get bucket policy")
         if current_tags_dict != expected_tags_dict:
             time.sleep(5)
@@ -580,6 +775,30 @@ def get_current_bucket_tags_dict(s3_client, bucket_name):
     return boto3_tag_list_to_ansible_dict(current_tags)
 
 
+def get_bucket_public_access(s3_client, bucket_name):
+    '''
+    Get current bucket public access block
+    '''
+    try:
+        bucket_public_access_block = s3_client.get_public_access_block(Bucket=bucket_name)
+        return bucket_public_access_block['PublicAccessBlockConfiguration']
+    except is_boto3_error_code('NoSuchPublicAccessBlockConfiguration'):
+        return {}
+
+
+def get_bucket_ownership_cntrl(s3_client, module, bucket_name):
+    '''
+    Get current bucket public access block
+    '''
+    if not module.botocore_at_least('1.18.11'):
+        return None
+    try:
+        bucket_ownership = s3_client.get_bucket_ownership_controls(Bucket=bucket_name)
+        return bucket_ownership['OwnershipControls']['Rules'][0]['ObjectOwnership']
+    except is_boto3_error_code(['OwnershipControlsNotFoundError', 'NoSuchOwnershipControls']):
+        return None
+
+
 def paginated_list(s3_client, **pagination_params):
     pg = s3_client.get_paginator('list_objects_v2')
     for page in pg.paginate(**pagination_params):
@@ -602,9 +821,9 @@ def destroy_bucket(s3_client, module):
     name = module.params.get("name")
     try:
         bucket_is_present = bucket_exists(s3_client, name)
-    except EndpointConnectionError as e:
+    except botocore.exceptions.EndpointConnectionError as e:
         module.fail_json_aws(e, msg="Invalid endpoint provided: %s" % to_text(e))
-    except (BotoCoreError, ClientError) as e:
+    except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
         module.fail_json_aws(e, msg="Failed to check bucket presence")
 
     if not bucket_is_present:
@@ -632,15 +851,15 @@ def destroy_bucket(s3_client, module):
                             ),
                             errors=resp['Errors'], response=resp
                         )
-        except (BotoCoreError, ClientError) as e:
+        except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
             module.fail_json_aws(e, msg="Failed while deleting bucket")
 
     try:
         delete_bucket(s3_client, name)
         s3_client.get_waiter('bucket_not_exists').wait(Bucket=name, WaiterConfig=dict(Delay=5, MaxAttempts=60))
-    except WaiterError as e:
+    except botocore.exceptions.WaiterError as e:
         module.fail_json_aws(e, msg='An error occurred waiting for the bucket to be deleted.')
-    except (BotoCoreError, ClientError) as e:
+    except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
         module.fail_json_aws(e, msg="Failed to delete bucket")
 
     module.exit_json(changed=True)
@@ -691,18 +910,32 @@ def main():
         versioning=dict(type='bool'),
         ceph=dict(default=False, type='bool'),
         encryption=dict(choices=['none', 'AES256', 'aws:kms']),
-        encryption_key_id=dict()
+        encryption_key_id=dict(),
+        public_access=dict(type='dict', options=dict(
+            block_public_acls=dict(type='bool', default=False),
+            ignore_public_acls=dict(type='bool', default=False),
+            block_public_policy=dict(type='bool', default=False),
+            restrict_public_buckets=dict(type='bool', default=False))),
+        delete_public_access=dict(type='bool', default=False),
+        object_ownership=dict(type='str', choices=['BucketOwnerPreferred', 'ObjectWriter']),
+        delete_object_ownership=dict(type='bool', default=False),
     )
 
     required_by = dict(
         encryption_key_id=('encryption',),
     )
 
+    mutually_exclusive = [
+        ['public_access', 'delete_public_access'],
+        ['delete_object_ownership', 'object_ownership']
+    ]
+
     module = AnsibleAWSModule(
-        argument_spec=argument_spec, required_by=required_by
+        argument_spec=argument_spec, required_by=required_by, mutually_exclusive=mutually_exclusive
     )
 
     region, ec2_url, aws_connect_kwargs = get_aws_connection_info(module, boto3=True)
+    validate_bucket_name(module, module.params["name"])
 
     if region in ('us-east-1', '', None):
         # default to US Standard region
@@ -730,15 +963,16 @@ def main():
     s3_client = get_s3_client(module, aws_connect_kwargs, location, ceph, s3_url)
 
     if s3_client is None:  # this should never happen
-        module.fail_json(msg='Unknown error, failed to create s3 connection, no information from boto.')
+        module.fail_json(msg='Unknown error, failed to create s3 connection, no information available.')
 
     state = module.params.get("state")
     encryption = module.params.get("encryption")
     encryption_key_id = module.params.get("encryption_key_id")
+    delete_object_ownership = module.params.get('delete_object_ownership')
+    object_ownership = module.params.get('object_ownership')
 
-    if not hasattr(s3_client, "get_bucket_encryption"):
-        if encryption is not None:
-            module.fail_json(msg="Using bucket encryption requires botocore version >= 1.7.41")
+    if delete_object_ownership or object_ownership:
+        module.require_botocore_at_least('1.18.11', reason='to manipulate bucket ownership controls')
 
     # Parameter validation
     if encryption_key_id is not None and encryption != 'aws:kms':
